from acai_aws.base.event import BaseRecordsEvent
from acai_aws.mq.record import Record


class Event(BaseRecordsEvent):

    def __init__(self, event, context=None, **kwargs):
        super().__init__(event, context, **kwargs)
        self._record_class = Record

    @property
    def raw_messages(self):
        return self._event.get('messages', [])

    @property
    def raw_records(self):
        return self.raw_messages



import unittest

from acai_aws.common.records.event import Event
from acai_aws.common.records.record import Record

from tests.mocks.common.mock_data_class import MockDataClass


class CommonRecordsTest(unittest.TestCase):
    event = {'key': 'value'}

    def test_event_accepts_event(self):
        event = Event(self.event)
        self.assertDictEqual(event.records[0].body, self.event)

    def test_event_accepts_event_list(self):
        event = Event([self.event])
        self.assertDictEqual(event.records[0].body, self.event)

    def test_event_returns_data_class(self):
        event = Event(self.event)
        event.data_class = MockDataClass
        self.assertTrue(isinstance(event.records[0], MockDataClass))
        self.assertTrue(isinstance(event.records[0].record, Record))

    def test_event_accepts_event_provides_raw_records(self):
        event = Event(self.event)
        self.assertDictEqual(event.raw_records[0], self.event)



import json
import unittest

from acai_aws.sns.record import Record

from tests.mocks.sns import mock_event


class SNSRecordTest(unittest.TestCase):
    basic_record = mock_event.get_basic()['Records'][0]

    def test_record_accepts_event(self):
        expected_attributes = {'Test': 'TestString', 'TestBinary': 'TestBinary'}
        record = Record(self.basic_record)
        self.assertEqual(record.version, self.basic_record['EventVersion'])
        self.assertEqual(record.subscription_arn, self.basic_record['EventSubscriptionArn'])
        self.assertEqual(record.source, self.basic_record['EventSource'])
        self.assertEqual(record.signature_version, self.basic_record['Sns']['SignatureVersion'])
        self.assertEqual(record.timestamp, self.basic_record['Sns']['Timestamp'])
        self.assertEqual(record.signature, self.basic_record['Sns']['Signature'])
        self.assertEqual(record.signing_cert_url, self.basic_record['Sns']['SigningCertUrl'])
        self.assertEqual(record.message_id, self.basic_record['Sns']['MessageId'])
        self.assertEqual(record.message, self.basic_record['Sns']['Message'])
        self.assertEqual(record.message_attributes, self.basic_record['Sns']['MessageAttributes'])
        self.assertDictEqual(record.attributes, expected_attributes)
        self.assertEqual(record.sns_type, self.basic_record['Sns']['Type'])
        self.assertEqual(record.unsubscribe_url, self.basic_record['Sns']['UnsubscribeUrl'])
        self.assertEqual(record.topic_arn, self.basic_record['Sns']['TopicArn'])
        self.assertEqual(record.subject, self.basic_record['Sns']['Subject'])
        self.assertDictEqual(record.body, json.loads(self.basic_record['Sns']['Message']))
        self.assertEqual(record.operation, record.UNKNOWN)

    def test_record_prints(self):
        try:
            record = Record(self.basic_record)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import unittest

from acai_aws.mq.event import Event
from acai_aws.mq.record import Record
from acai_aws.common.records.exception import RecordException

from tests.mocks.mq import mock_event
from tests.mocks.mq.mock_data_class import MockMQDataClass


class MQEventTest(unittest.TestCase):
    basic_event = mock_event.get_basic()
    schema_path = 'tests/mocks/mq/openapi.yml'
    expected_body = {
        'lang': 'en-us',
        'sms': True,
        'email': True,
        'push': True
    }

    def test_event_accepts_event(self):
        event = Event(self.basic_event)
        self.assertEqual(event.context, None)
        self.assertEqual(event.data_class, None)
        self.assertDictEqual(event.event, self.basic_event)
        self.assertEqual(len(event.records), len(self.basic_event['messages']))

    def test_event_raw_records(self):
        event = Event(self.basic_event)
        self.assertCountEqual(event.raw_records, self.basic_event['messages'])

    def test_event_returns_record_event(self):
        event = Event(self.basic_event)
        self.assertTrue(isinstance(event.records[0], Record))

    def test_event_returns_data_class(self):
        event = Event(self.basic_event)
        event.data_class = MockMQDataClass
        self.assertTrue(isinstance(event.records[0], MockMQDataClass))
        self.assertTrue(isinstance(event.records[0].record, Record))

    def test_event_validate_record_body_with_schema_file(self):
        event = Event(self.basic_event, schema=self.schema_path, required_body='v1-mq-body')
        self.assertDictEqual(event.records[0].body, self.expected_body)

    def test_event_validate_filters_out_record_body_with_schema_file(self):
        event = Event(self.basic_event, schema=self.schema_path, required_body='v1-mq-body-wrong')
        self.assertEqual(len(event.records), 0)

    def test_event_validate_raises_exception_record_body_with_schema_file(self):
        try:
            event = Event(self.basic_event, schema=self.schema_path, required_body='v1-mq-body-wrong', raise_body_error=True)
            print(event.records)
            self.assertTrue(False)
        except RecordException as record_error:
            self.assertTrue(isinstance(record_error, RecordException))

    def test_event_validate_record_body_with_schema_dict(self):
        schema = {
            '$id': 'https://example.com/person.schema.json',
            '$schema': 'https://json-schema.org/draft/2020-12/schema',
            'title': 'Person',
            'type': 'object',
            "properties": {
                'lang': {
                    'type': 'string'
                },
                'sms': {
                    'type': 'boolean'
                },
                'email': {
                    'type': 'boolean'
                },
                'push': {
                    'type': 'boolean'
                }
            }
        }
        event = Event(self.basic_event, schema=self.schema_path, required_body=schema)
        self.assertDictEqual(event.records[0].body, self.expected_body)

    def test_event_validate_raises_error_record_body_with_schema_dict(self):
        schema = {
            '$id': 'https://example.com/person.schema.json',
            '$schema': 'https://json-schema.org/draft/2020-12/schema',
            'title': 'Person',
            'type': 'object',
            "properties": {
                'lang': {
                    'type': 'integer'
                },
                'sms': {
                    'type': 'boolean'
                },
                'email': {
                    'type': 'boolean'
                },
                'push': {
                    'type': 'boolean'
                }
            }
        }
        try:
            event = Event(self.basic_event, schema=self.schema_path, required_body=schema, raise_body_error=True)
            print(event.records)
            self.assertTrue(False)
        except RecordException as record_error:
            self.assertTrue(isinstance(record_error, RecordException))

    def test_event_validate_filters_out_record_body_with_schema_dict(self):
        schema = {
            '$id': 'https://example.com/person.schema.json',
            '$schema': 'https://json-schema.org/draft/2020-12/schema',
            'title': 'Person',
            'type': 'object',
            "properties": {
                'lang': {
                    'type': 'integer'
                },
                'sms': {
                    'type': 'boolean'
                },
                'email': {
                    'type': 'boolean'
                },
                'push': {
                    'type': 'boolean'
                }
            }
        }
        event = Event(self.basic_event, schema=self.schema_path, required_body=schema)
        self.assertEqual(len(event.records), 0)

    def test_event_print(self):
        event = Event(self.basic_event)
        try:
            print(event)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import unittest

from acai_aws.common.records.exception import EventTimeOutException

from acai_aws.common.records.event import Event as CommonEvent
from acai_aws.documentdb.event import Event as DocumentDBEvent
from acai_aws.dynamodb.event import Event as DynamoDBEvent
from acai_aws.firehose.event import Event as FirehoseEvent
from acai_aws.kinesis.event import Event as KinesisEvent
from acai_aws.msk.event import Event as MSKEvent
from acai_aws.mq.event import Event as MQEvent
from acai_aws.s3.event import Event as S3Event
from acai_aws.sns.event import Event as SNSEvent
from acai_aws.sqs.event import Event as SQSEvent

from tests.mocks.documentdb import mock_event as mock_docdb
from tests.mocks.dynamodb import mock_event as mock_ddb
from tests.mocks.firehose import mock_event as mock_firehose
from tests.mocks.kinesis import mock_event as mock_kinesis
from tests.mocks.msk import mock_event as mock_msk
from tests.mocks.mq import mock_event as mock_mq
from tests.mocks.s3 import mock_event as mock_s3
from tests.mocks.sns import mock_event as mock_sns
from tests.mocks.sqs import mock_event as mock_sqs

from tests.mocks.common.mock_functions import mock_func_verbose, mock_func_timeout


class CommonRequirementsTest(unittest.TestCase):
    context = None
    unknown_event = {'unknown': 'value'}
    docdb_event = mock_docdb.get_basic()
    ddb_event = mock_ddb.get_created_event()
    firehose_event = mock_firehose.get_basic()
    kinesis_event = mock_kinesis.get_basic()
    msk_event = mock_msk.get_basic()
    mq_event = mock_mq.get_basic()
    s3_event = mock_s3.get_basic()
    sns_event = mock_sns.get_basic()
    sqs_event = mock_sqs.get_basic()

    def test_decorator_with_unknown_event(self):
        result = mock_func_verbose(self.unknown_event, self.context)
        self.assertTrue(isinstance(result, CommonEvent))

    def test_decorator_with_docdb_event(self):
        result = mock_func_verbose(self.docdb_event, self.context)
        self.assertTrue(isinstance(result, DocumentDBEvent))

    def test_decorator_with_ddb_event(self):
        result = mock_func_verbose(self.ddb_event, self.context)
        self.assertTrue(isinstance(result, DynamoDBEvent))

    def test_decorator_with_firehose_event(self):
        result = mock_func_verbose(self.firehose_event, self.context)
        self.assertTrue(isinstance(result, FirehoseEvent))

    def test_decorator_with_kinesis_event(self):
        result = mock_func_verbose(self.kinesis_event, self.context)
        self.assertTrue(isinstance(result, KinesisEvent))

    def test_decorator_with_msk_event(self):
        result = mock_func_verbose(self.msk_event, self.context)
        self.assertTrue(isinstance(result, MSKEvent))

    def test_decorator_with_mq_event(self):
        result = mock_func_verbose(self.mq_event, self.context)
        self.assertTrue(isinstance(result, MQEvent))

    def test_decorator_with_s3_event(self):
        result = mock_func_verbose(self.s3_event, self.context)
        self.assertTrue(isinstance(result, S3Event))

    def test_decorator_with_sns_event(self):
        result = mock_func_verbose(self.sns_event, self.context)
        self.assertTrue(isinstance(result, SNSEvent))

    def test_decorator_with_sqs_event(self):
        result = mock_func_verbose(self.sqs_event, self.context)
        self.assertTrue(isinstance(result, SQSEvent))

    def test_decorator_with_timeout(self):
        try:
            mock_func_timeout(self.sqs_event, self.context)
            self.assertTrue(False)
        except EventTimeOutException as error:
            self.assertTrue(isinstance(error, EventTimeOutException))



from datetime import datetime

from acai_aws.common import logger
from acai_aws.base.record import BaseRecord


class Record(BaseRecord):

    def __init__(self, event):
        super().__init__(event)
        self._event = event.get('event')
        self.mongo_operations = {
            'create': self.CREATED,
            'createIndexes': self.CREATED,
            'delete': self.DELETED,
            'drop': self.DELETED,
            'dropDatabase': self.DELETED,
            'dropIndexes': self.DELETED,
            'insert': self.CREATED,
            'invalidate': self.DELETED,
            'modify': self.UPDATED,
            'rename': self.UPDATED,
            'update': self.UPDATED,
            'shardCollection': self.CREATED
        }

    @property
    def event_id(self):
        return self._event.get('_id', {}).get('_data')

    @property
    def cluster_time(self):
        time_stamp = self._event.get('clusterTime', {}).get('$timestamp', {})
        time = time_stamp.get('t')
        zone = time_stamp.get('i')
        return datetime.fromtimestamp(float(f'{time}.{zone}')).isoformat()

    @property
    def document_key(self):
        return self._event.get('documentKey', {}).get('_id', {}).get('$oid')

    @property
    def full_document(self):
        return self._event.get('fullDocument', {})

    @property
    def body(self):
        body = self.full_document.copy()
        body['id'] = self.full_document['_id']['$oid']
        del body['_id']
        return body

    @property
    def operation(self):
        try:
            return self.mongo_operations[self.change_event]
        except Exception as error:
            logger.log(level='ERROR', log={'event': self._event, 'error': error})
            return self.UNKNOWN

    @property
    def change_event(self):
        return self._event.get('operationType', 'unknown')

    @property
    def db(self):
        return self._event.get('ns', {}).get('db')

    @property
    def collection(self):
        return self._event.get('ns', {}).get('coll')

    def __str__(self):
        return str({
            'event_id': self.event_id,
            'cluster_time': self.cluster_time,
            'document_key': self.document_key,
            'operation': self.operation,
            'full_document': self.full_document,
            'db': self.db,
            'collection': self.collection
        })



import unittest

from acai_aws.generic.event import Event


class GenericEventTest(unittest.TestCase):
    basic_event = {'key': 'value'}

    def test_event_accepts_event(self):
        event = Event(self.basic_event)
        self.assertDictEqual(self.basic_event, event.body)



import base64
import urllib

import xmltodict

from acai_aws.common.json_helper import JsonHelper


class Request:

    def __init__(self, event, lambda_context=None, timeout=None):
        self.__event = event
        self.lambda_context = lambda_context
        self.__timeout = timeout
        self.__body = event['body'] if event.get('body') is not None else {}
        self.__route = event['path'] if event.get('path') is not None else ''
        self.__path_params = event['pathParameters'] if event.get('pathParameters') is not None else ''
        self.__request_context = event['requestContext'] if event.get('requestContext') is not None else {}
        self.__domain = event.get('requestContext', {}).get('domainName', '')
        self.__stage = event.get('requestContext', {}).get('stage', '')
        self.__context = {}
        self.__parsers = {
            'application/json': 'json',
            'application/graphql': 'graphql',
            'application/x-www-form-urlencoded': 'form',
            'multipart/form-data': 'raw',
            'application/xml': 'xml',
            'text/xml': 'xml',
            'raw': 'raw'
        }

    @property
    def cookies(self):
        if self.__event.get('headers', {}).get('cookie'):
            cookies = self.__event.get('headers', {}).get('cookie')
        else:
            cookies = ';'.join(self.__event.get('cookies', []))
        return cookies

    @property
    def protocol(self):
        if self.__request_context.get('protocol'):
            protocol = self.__request_context.get('protocol', 'http')
        else:
            protocol = self.__request_context.get('http', {}).get('protocol', 'http')
        return 'https' if 'https' in protocol.lower() else 'http'

    @property
    def content_type(self):
        return self.headers.get('content-type', '').split(';')[0]

    @property
    def host_url(self):
        return f'{self.protocol}://{self.__domain}'

    @property
    def domain(self):
        return self.__domain

    @property
    def stage(self):
        return self.__stage

    @property
    def method(self):
        if self.__event.get('httpMethod') is not None:
            return self.__event['httpMethod'].lower()
        return ''

    @property
    def resource(self):
        if self.__event.get('resource') is not None:
            return self.__event['resource']
        return ''

    @property
    def path(self):
        if self.__event.get('path') is not None:
            return self.__event['path']
        return ''

    @property
    def route(self):
        if self.__route and self.__route[0] != '/':
            return f'/{self.__route}'
        return self.__route

    @route.setter
    def route(self, route):
        self.__route = route

    @property
    def authorizer(self):
        if self.__event.get('isOffline') is not None:
            return self.headers
        return self.__request_context.get('authorizer', self.headers)

    @property
    def headers(self):
        headers = {k.lower(): v for k, v in self.__event.get('headers', {}).items()}
        return headers

    @property
    def body(self):
        try:
            content_type = self.headers.get('content-type', '').split(';')[0]
            parser = self.__parsers.get(content_type, 'raw')
            return getattr(self, parser)
        except Exception as error:
            print(error)
            return self.__body

    @property
    def json(self):
        return JsonHelper.decode(self.__body, True)

    @property
    def form(self):
        return dict(urllib.parse.parse_qsl(self.__body))

    @property
    def xml(self):
        return xmltodict.parse(self.__body)

    @property
    def graphql(self):
        try:
            request = base64.b64decode(self.__body).decode('utf-8')
        except Exception as error:
            print(error)
            request = self.__body
        return JsonHelper.decode(request)

    @property
    def raw(self):
        return self.__body

    @property
    def params(self):
        return {'query': self.query_params, 'path': self.path_params}

    @property
    def query_params(self):
        if self.__event.get('queryStringParameters') is not None:
            return self.__event['queryStringParameters']
        return {}

    @property
    def path_params(self):
        return self.__path_params

    @path_params.setter
    def path_params(self, path_params):
        key, value = path_params
        self.__path_params[key] = value

    @property
    def context(self):
        return self.__context

    @context.setter
    def context(self, context):
        self.__context = context

    @property
    def event(self):
        return self.__event

    @property
    def timeout(self):
        return self.__timeout

    @property
    def full(self):
        return {
            'method': self.method,
            'resource': self.resource,
            'headers': self.headers,
            'authorizer': self.authorizer,
            'params': self.params,
            'body': self.body,
            'context': self.context
        }

    def clear_path_params(self):
        self.__path_params = {}

    def __str__(self):
        return JsonHelper.encode(self.full)
